@@ -133,25 +133,37 @@ KIO::SimpleJob* KonqOperations::mkdir( QWidget *parent, const KUrl & url )
 
 void KonqOperations::doPaste( QWidget * parent, const KUrl & destUrl, const QPoint &pos )
 {
-    QClipboard* clipboard = QApplication::clipboard();
+    (void) KonqOperations::doPasteV2( parent, destUrl, pos );
+}
+
+KonqOperations *KonqOperations::doPasteV2(QWidget *parent, const KUrl &destUrl, const QPoint &pos)
+{
+    QClipboard *clipboard = QApplication::clipboard();
     const QMimeData *data = clipboard->mimeData();
     const bool move = KonqMimeData::decodeIsCutSelection(data);
 
-    KIO::Job *job = KIO::pasteClipboard( destUrl, parent, move );
+    KIO::Job *job = KIO::pasteClipboard(destUrl, parent, move);
     if (job) {
-        KonqOperations * op = new KonqOperations( parent );
-        KIOPasteInfo * pi = new KIOPasteInfo;
+        KonqOperations *op = new KonqOperations(parent);
+        KIOPasteInfo *pi = new KIOPasteInfo;
         pi->mousePos = pos;
-        op->setPasteInfo( pi );
-        KIO::CopyJob * copyJob = qobject_cast<KIO::CopyJob *>(job);
+        op->setPasteInfo(pi);
+        KIO::CopyJob *copyJob = qobject_cast<KIO::CopyJob*>(job);
         if (copyJob) {
-            op->setOperation( job, move ? MOVE : COPY, copyJob->destUrl() );
-            KIO::FileUndoManager::self()->recordJob( move ? KIO::FileUndoManager::Move : KIO::FileUndoManager::Copy, KUrl::List(), destUrl, job );
-        } else if (KIO::SimpleJob* simpleJob = qobject_cast<KIO::SimpleJob *>(job)) {
+            op->setOperation(job, move ? MOVE : COPY, copyJob->destUrl());
+            KIO::FileUndoManager::self()->recordJob(move ? KIO::FileUndoManager::Move : KIO::FileUndoManager::Copy, KUrl::List(), destUrl, job);
+            connect(copyJob, SIGNAL(copyingDone(KIO::Job*,KUrl,KUrl,time_t,bool,bool)),
+                    op, SLOT(slotCopyingDone(KIO::Job*,KUrl,KUrl)));
+            connect(copyJob, SIGNAL(copyingLinkDone(KIO::Job*,KUrl,QString,KUrl)),
+                    op, SLOT(slotCopyingLinkDone(KIO::Job*,KUrl,QString,KUrl)));
+        } else if (KIO::SimpleJob *simpleJob = qobject_cast<KIO::SimpleJob*>(job)) {
             op->setOperation(job, PUT, simpleJob->url());
             KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Put, KUrl::List(), simpleJob->url(), job);
         }
+        return op;
     }
+
+    return 0;
 }
 
 void KonqOperations::copy( QWidget * parent, Operation method, const KUrl::List & selectedUrls, const KUrl& destUrl )
@@ -177,6 +189,11 @@ void KonqOperations::copy( QWidget * parent, Operation method, const KUrl::List
     else
         job = KIO::copy( selectedUrls, destUrl );
 
+    connect(job, SIGNAL(copyingDone(KIO::Job*,KUrl,KUrl,time_t,bool,bool)),
+            op, SLOT(slotCopyingDone(KIO::Job*,KUrl,KUrl)));
+    connect(job, SIGNAL(copyingLinkDone(KIO::Job*,KUrl,QString,KUrl)),
+            op, SLOT(slotCopyingLinkDone(KIO::Job*,KUrl,QString,KUrl)));
+
     op->setOperation( job, method, destUrl );
 
     KIO::FileUndoManager::self()->recordCopyJob(job);
@@ -636,25 +653,42 @@ void KonqOperations::doDropFileCopy()
         KIO::FileUndoManager::self()->recordJob(
             m_method == TRASH ? KIO::FileUndoManager::Trash : KIO::FileUndoManager::Move,
             lst, m_destUrl, job );
-        return; // we still have stuff to do -> don't delete ourselves
+        break;
     case Qt::CopyAction :
         job = KIO::copy( lst, m_destUrl );
         job->setMetaData( m_info->metaData );
         setOperation( job, COPY, m_destUrl );
         KIO::FileUndoManager::self()->recordCopyJob(job);
-        return;
+        break;
     case Qt::LinkAction :
         kDebug(1203) << "lst.count=" << lst.count();
         job = KIO::link( lst, m_destUrl );
         job->setMetaData( m_info->metaData );
         setOperation( job, LINK, m_destUrl );
         KIO::FileUndoManager::self()->recordCopyJob(job);
-        return;
+        break;
     default : kError(1203) << "Unknown action " << (int)action << endl;
     }
+    if (job) {
+        connect(job, SIGNAL(copyingDone(KIO::Job*,KUrl,KUrl,time_t,bool,bool)),
+                this, SLOT(slotCopyingDone(KIO::Job*,KUrl,KUrl)));
+        connect(job, SIGNAL(copyingLinkDone(KIO::Job*,KUrl,QString,KUrl)),
+                this, SLOT(slotCopyingLinkDone(KIO::Job*,KUrl,QString,KUrl)));
+        return; // we still have stuff to do -> don't delete ourselves
+    }
     deleteLater();
 }
 
+void KonqOperations::slotCopyingDone( KIO::Job*, const KUrl&, const KUrl &to)
+{
+    m_createdUrls << to;
+}
+
+void KonqOperations::slotCopyingLinkDone(KIO::Job*, const KUrl&, const QString&, const KUrl &to)
+{
+    m_createdUrls << to;
+}
+
 void KonqOperations::_addPluginActions(QList<QAction*>& pluginActions,const KUrl& destination, const KFileItemListProperties& info)
 {
     kDebug(1203);
@@ -716,16 +750,21 @@ static QString translatePath( QString path ) // krazy:exclude=passbyvalue
 }
 
 void KonqOperations::rename( QWidget * parent, const KUrl & oldurl, const KUrl& newurl )
+{
+    renameV2(parent, oldurl, newurl);
+}
+
+KonqOperations *KonqOperations::renameV2( QWidget * parent, const KUrl & oldurl, const KUrl& newurl )
 {
     kDebug(1203) << "oldurl=" << oldurl << " newurl=" << newurl;
     if ( oldurl == newurl )
-        return;
+        return 0;
 
     KUrl::List lst;
     lst.append(oldurl);
     KIO::Job * job = KIO::moveAs( oldurl, newurl, oldurl.isLocalFile() ? KIO::HideProgressInfo : KIO::DefaultFlags );
     KonqOperations * op = new KonqOperations( parent );
-    op->setOperation( job, MOVE, newurl );
+    op->setOperation( job, RENAME, newurl );
     KIO::FileUndoManager::self()->recordJob( KIO::FileUndoManager::Rename, lst, newurl, job );
     // if moving the desktop then update config file and emit
     if ( oldurl.isLocalFile() && oldurl.toLocalFile( KUrl::AddTrailingSlash ) == KGlobalSettings::desktopPath() )
@@ -738,6 +777,8 @@ void KonqOperations::rename( QWidget * parent, const KUrl & oldurl, const KUrl&
         g.writeEntry( "XDG_DESKTOP_DIR", QString("\"" + translatePath( newurl.path() ) + "\"") );
         KGlobalSettings::self()->emitChange(KGlobalSettings::SettingsChanged, KGlobalSettings::SETTINGS_PATHS);
     }
+
+    return op;
 }
 
 void KonqOperations::setOperation( KIO::Job * job, Operation method, const KUrl & dest )
@@ -804,25 +845,60 @@ void KonqOperations::slotStatResult( KJob * job )
         deleteLater();
 }
 
-void KonqOperations::slotResult( KJob * job )
+void KonqOperations::slotResult(KJob *job)
 {
-    if (job && job->error())
-    {
-        static_cast<KIO::Job*>( job )->ui()->showErrorMessage();
+    bool jobFailed = false;
+    if (job && job->error()) {
+        static_cast<KIO::Job*>(job)->ui()->showErrorMessage();
+        jobFailed = true;
     }
-    if ( m_method == EMPTYTRASH || m_method == RESTORE ) {
-        // Update views showing trash:/
-        org::kde::KDirNotify::emitFilesAdded( "trash:/" );
+
+    switch (m_method) {
+    case PUT: {
+            KIO::SimpleJob *simpleJob = qobject_cast<KIO::SimpleJob*>(job);
+            if (simpleJob && !jobFailed) {
+                m_createdUrls << simpleJob->url();
+            }
+        }
+        break;
+    case EMPTYTRASH:
+    case RESTORE:
+        // Update konq windows opened on trash:/
+        org::kde::KDirNotify::emitFilesAdded("trash:/"); // yeah, files were removed, but we don't know which ones...
+        break;
+    case RENAME: {
+            KIO::CopyJob *renameJob = qobject_cast<KIO::CopyJob*>(job);
+            if (renameJob && jobFailed) {
+                const KUrl oldUrl = renameJob->srcUrls().first();
+                const KUrl newUrl = renameJob->destUrl();
+                emit renamingFailed(oldUrl, newUrl);
+            }
+        }
+        break;
+    default:
+        break;
     }
+
+    if (!m_createdUrls.isEmpty()) {
+        // Inform the application about all created urls
+        emit aboutToCreate(m_createdUrls);
+        m_createdUrls.clear();
+    }
+
     deleteLater();
 }
 
 void KonqOperations::rename( QWidget * parent, const KUrl & oldurl, const QString & name )
+{
+    renameV2(parent, oldurl, name);
+}
+
+KonqOperations *KonqOperations::renameV2( QWidget * parent, const KUrl & oldurl, const QString & name )
 {
     KUrl newurl( oldurl );
     newurl.setPath( oldurl.directory( KUrl::AppendTrailingSlash ) + name );
     kDebug(1203) << "KonqOperations::rename("<<name<<") called. newurl=" << newurl;
-    rename( parent, oldurl, newurl );
+    return renameV2( parent, oldurl, newurl );
 }
 
 // Duplicated in libkfile's KDirOperator
